'
' Card Drawing scripts
' These allow you to set a base deck of cards, and then draw from it.  
' Stores the deck information using GetField & SetField methods
'

#commandset Card Drawing
#event OnStartup CardDrawingInitDeck
#register /shuffle CardDrawingShuffle Shuffles the deck when there's not enough cards to draw
#register /Draw CardDrawingDraw Draws one or more cards from the deck.  Usage "/draw" For one card, "/draw 3" For 3 cards, etc. 

Sub CardDrawingInitDeck
	'This subroutine is called on startup to set a standard 
	'deck of cards including both Jokers.
	CardDrawingSetDeck "2H;2D;2C;2S;3H;3D;3C;3S;4H;4D;4C;4S;5H;5D;5C;5S;6H;6D;6C;6S;7H;7D;7C;7S;8H;8D;8C;8S;9H;9D;9C;9S;10H;10D;10C;10S;JH;JD;JC;JS;QH;QD;QC;QS;KH;KD;KC;KS;AH;AD;AC;AS;RedJoker;BlackJoker"
End Sub

Sub CardDrawingSetDeck( sDeck)
	s = GetCurrentSession()
        'This sets two new permanent fields for the session 
        'BaseDeck is all the cards in a deck
        'CurrentDeck is the cards remaining in the current deck
	s.SetField "BaseDeck", Trim( sDeck)
	s.SetField "CurrentDeck", Trim( sDeck)
End Sub


Sub CardDrawingShuffle
	s = GetCurrentSession()
	cs = GetChatSession()
	ci = cs.GetLastItem()	
	ci.Action = "Shuffles the deck"
	ci.Text = ""
	addlog "Shuffling deck" 
	s.SetField "CurrentDeck", s.GetField("BaseDeck")
End Sub



Sub CardDrawingDraw
	s = GetCurrentSession()
	cs = GetChatSession()
	ci = cs.GetLastItem()

	nReps = 1
	If Len( Trim( ci.Text)) > 0 Then
		nReps = CInt( ci.Text) + 0
	End If
	
	If nReps < 1 Then nReps = 1

	'clear the text output
	ci.text = ""

	'get the max number of reps, so you cant draw more than one deck at a time
	deck = s.GetField( "CurrentDeck")
	'a is an array created using the split command.
	a = split( deck, ";")
	'n is the last element number in this 0 based array.
	n = UBound( a) + 1

	'sets the maximum number of repetitions to the number in the array.
	If nReps > n Then
		ci.Text = ci.Text & "There are not enough cards in the deck. Only " & n & " cards left. Deck will be shuffled.<br>"

	End If
    	
	If Len(deck) > 0 Then
		ci.Action = "Draws"

		Dim i
		For i = 1 to nReps 
			'check to see if the deck needs shuffled
			If i > n Then
				addlog "Shuffling deck" 
				s.SetField "CurrentDeck", s.GetField("BaseDeck")
				ci.Text = ci.Text & "<br> Deck Shuffled <br> "
			End If
			CardDraw = DrawACard
			ci.Text = ci.Text & "<b>" & CardDraw & "</b> "
		Next

	Else
		ci.Action = "Draws"
		ci.Text = "The Deck is empty!"		
	End If
End Sub

Function DrawACard( )
	s = GetCurrentSession()
	deck = s.GetField( "CurrentDeck")
	a = split( deck, ";")
	n = UBound( a) + 1

	If n > 0 Then
		'Randomize the random number generator
		Randomize

		'randomly select a remaining card
		p = Int(n * Rnd() )
		pick = a(p)
		'erase the deck variable		
		deck = ""
		'relace each card in the deck variable, other than the one chosen.
		For j = 0 to n-1
			If j <> p Then
				deck = deck & a(j) & ";"
			End If
		Next
		'get rid of the trailing semicolon cuz it will mess up the next Split()
		If Right( deck, 1) = ";" Then deck = Left( deck, Len( deck) -1)
		
		'Set the current deck to be updated with the right number of cards
		s.SetField "CurrentDeck", deck
	Else
		pick = "No Cards in Deck."
	End If

	DrawACard=pick
End Function


